#!/usr/bin/env python3

import re, requests, socket, ssl, struct, sys, urllib3, warnings


def check_version(scheme, host, port):
    warnings.filterwarnings(
        "ignore", category=urllib3.exceptions.InsecureRequestWarning
    )
    try:
        resp = requests.get(
            f"{scheme}://{host}:{port}/dana-na/auth/url_admin/welcome.cgi?type=inter",
            verify=False,
            timeout=10,
        )
        resp.raise_for_status()
        match = re.search(r'<PARAM NAME="ProductVersion"\s+VALUE="([0-9.]+)', resp.text)
        if match:
            version = match.group(1)
            if version.startswith("22.7.2.") or version in [
                "22.7.1.907",  # IPS 22.7.1R1
                "22.7.1.1321",  # IPS 22.7.1R1.1
                "22.7.1.1485",  # IPS 22.7.1R1.2
            ]:
                return True
            else:
                print(f"{scheme}://{host}:{port}: Version {version} is not affected")
                return False
    except:
        print(f"{scheme}://{host}:{port}: No response from web server")
        return False


def p32(x):
    return struct.pack("<I", x)


def negotiate_ifttls(scheme, host, port):
    try:
        ctx = ssl._create_unverified_context()
        s = ctx.wrap_socket(socket.create_connection((host, port), timeout=3))
    except:
        print(f"{scheme}://{host}:{port}: Failed to create TLS socket connection")
        exit()
    req = b"GET / HTTP/1.1\r\n"
    req += f"Host: {host}\r\n".encode()
    req += b"User-Agent: BishopFox\r\n"
    req += b"Content-Type: EAP\r\n"
    req += b"Upgrade: IF-T/TLS 1.0\r\n"
    req += b"Content-Length: 0\r\n"
    req += b"\r\n"
    s.send(req)
    resp = s.recv(1024)
    if b"HTTP/1.1 101 Switching Protocols" not in resp:
        print(f"{scheme}://{host}:{port}: Server does not support IF-T/TLS")
        exit()
    return s


def send_ift(s, vendor, ptype, seqno, data):
    hdr = struct.pack(">IIII", vendor, ptype, len(data) + 16, seqno)
    s.send(hdr + data)


def rcv_ift(s):
    vendor, ptype, length, seqno = struct.unpack(">IIII", s.read(16))
    data = s.read(length - 16)
    return vendor, ptype, data


def main():
    # Parse input
    if len(sys.argv) < 1:
        print("Usage: scan-cve-2025-0282.py <SCHEME://HOST[:PORT]>")
        exit()
    parts = sys.argv[1].split(":")
    scheme = parts[0]
    host = parts[1].lstrip("/")
    if len(parts) == 2:
        port = 443
    else:
        port = int(parts[2])
    if scheme.lower() != "https":
        print(f"{scheme}://{host}:{port}: Target scheme is not https")
        exit()

    # Check version
    if not check_version(scheme, host, port):
        exit()

    # Create socket connection
    s = negotiate_ifttls(scheme, host, port)

    # Send client version request
    send_ift(s, 0x5597, 1, 0, b"\x00\x01\x02\x02")

    # Receive version response
    vendor, ptype, data = rcv_ift(s)

    # Receive auth challenge
    vendor, ptype, data = rcv_ift(s)

    # Send auth response
    payload = b"clientHostname=BishopFox"
    payload += b" clientIp=" + b"A" * 0x40
    payload += b"\n\0"
    send_ift(s, 0xA4C, 0x88, 1, payload)
    send_ift(s, 0xA4C, 0x88, 2, b"anonymous\n\0")

    # Evaluate vulnerability status
    try:
        vendor, ptype, data = rcv_ift(s)
        if vendor == 0xA4C and ptype == 0x93:
            print(f"{scheme}://{host}:{port}: Patched")
        else:
            print(
                f"{scheme}://{host}:{port}: Unexpected response: vendor={hex(vendor)}, type={hex(ptype)}, data={data.hex()}"
            )
    except socket.timeout:
        print(f"{scheme}://{host}:{port}: Vulnerable")


if __name__ == "__main__":
    main()
